-- Table to keep track of which players have received their equipment
local equipped_players = {}

-- Add these functions at the top of your file
local function is_wearing_jareds_armor(player)
    local armor_inventory = player.get_inventory(defines.inventory.character_armor)
    if not armor_inventory or armor_inventory.is_empty() then return false end
    local armor = armor_inventory[1]
    return armor and armor.valid and armor.name == "jareds-power-armor"
end

-- local function update_robot_status(player)
--     if is_wearing_jareds_armor(player) then
--         player.force.technologies["jareds-robots"].enabled = true
--     else
--         player.force.technologies["jareds-robots"].enabled = false
--     end
-- end

-- Modify your existing update_player_modifiers function
local function update_player_modifiers(player)
    if not player or not player.character then return end

    if is_wearing_jareds_armor(player) then
        player.character_running_speed_modifier = 0.9  -- 90% speed increase
        -- update_robot_status(player)
    else
        player.character_running_speed_modifier = 0
        -- update_robot_status(player)
    end
end


-- Your existing event handlers remain the same
script.on_event(defines.events.on_player_armor_inventory_changed, function(event)
    local player = game.get_player(event.player_index)
    update_player_modifiers(player)
end)

script.on_event(defines.events.on_player_respawned, function(event)
    local player = game.get_player(event.player_index)
    update_player_modifiers(player)
end)

-- Give Jareds Armor to the player on first move
script.on_event(defines.events.on_player_changed_position, function(event)
    local player = game.get_player(event.player_index)
    if not player or equipped_players[player.index] then
        return
    end

    local character = player.character
    if not character then
        log("Character not found for player " .. player.index .. " on first move")
        return
    end

    local armor_inventory = character.get_inventory(defines.inventory.character_armor)
    if not armor_inventory then
        log("Armor inventory not found for player " .. player.index .. " on first move")
        return
    end
    if armor_inventory.can_insert({name="jareds-power-armor", count=1}) then
        local inserted = armor_inventory.insert({name="jareds-power-armor", count=1})
        if inserted > 0 then
            
            -- Get the actual armor item
            local armor = armor_inventory[1]
            if armor and armor.valid and armor.grid then
                local grid = armor.grid
                
                -- Insert equipment into the armor
                grid.put({name="personal-roboport-mk2-equipment", position={0,0}})
                grid.put({name="personal-roboport-mk2-equipment", position={2,0}})
                
                -- Charge all equipment to full
                for _, equipment in pairs(grid.equipment) do
                    if equipment.max_energy > 0 then
                        equipment.energy = equipment.max_energy
                    end
                end
            end
            
            -- Add custom robots to main inventory
            local main_inventory = character.get_main_inventory()
            if main_inventory and main_inventory.can_insert({name="jareds-robots", count=50}) then
                main_inventory.insert({name="jareds-robots", count=50})
            end
            
            -- Mark this player as equipped
            equipped_players[player.index] = true
        end
    end
    update_player_modifiers(player)
end)

-- Clean up the equipped_players table when a player leaves
script.on_event(defines.events.on_player_left_game, function(event)
    equipped_players[event.player_index] = nil
end)